export declare type TRUE = {
    value: true;
};
export declare type FALSE = {
    value: false;
};
export declare type BOOLEAN = TRUE | FALSE;
export declare type PropertySpecs = {
    [key: string]: Spec;
};
declare type StringSpec = {
    target: "string";
    nullable: BOOLEAN;
    optional: BOOLEAN;
};
declare type NumberSpec = {
    target: "number";
    nullable: BOOLEAN;
    optional: BOOLEAN;
};
declare type BooleanSpec = {
    target: "boolean";
    nullable: BOOLEAN;
    optional: BOOLEAN;
};
declare type UnknownSpec = {
    target: "unknown";
    nullable: BOOLEAN;
    optional: BOOLEAN;
};
export declare type ArraySpec = {
    target: "array";
    element: Spec;
    nullable: BOOLEAN;
    optional: BOOLEAN;
};
export declare type ObjectSpec = {
    target: "object";
    entries: {
        [key: string]: Spec;
    };
    nullable: BOOLEAN;
    optional: BOOLEAN;
};
export declare type Spec = StringSpec | NumberSpec | BooleanSpec | ObjectSpec | ArraySpec | UnknownSpec;
export declare type Json<T extends Spec> = T extends StringSpec ? string | Optionality<T> | Nullability<T> : T extends NumberSpec ? number | Optionality<T> | Nullability<T> : T extends BooleanSpec ? boolean | Optionality<T> | Nullability<T> : T extends UnknownSpec ? unknown | Optionality<T> | Nullability<T> : T extends ArraySpec ? Json<T["element"]>[] | Optionality<T> | Nullability<T> : T extends ObjectSpec ? {
    [P in keyof T["entries"]]: Json<T["entries"][P]>;
} | Optionality<T> | Nullability<T> : never;
declare type Optionality<T extends Spec> = T["optional"] extends TRUE ? undefined : never;
declare type Nullability<T extends Spec> = T["nullable"] extends TRUE ? null : never;
export {};
